from .db import Database

class Model:
    @classmethod
    def create_table(cls):
        fields = []
        for name, field in cls.__dict__.items():
            if isinstance(field, Field):
                col = f"{name} {field.type}"
                if field.primary_key:
                    col += " PRIMARY KEY"
                if field.foreign_key:
                    col += f" REFERENCES {field.foreign_key}(id)"
                fields.append(col)
        query = f"CREATE TABLE IF NOT EXISTS {cls.__name__.lower()} ({', '.join(fields)})"
        Database.execute(query)

    def save(self):
        fields = [k for k in self.__class__.__dict__ if isinstance(getattr(self, k, None), (int, str, float))]
        values = [getattr(self, f) for f in fields]
        placeholders = ", ".join(["%s"] * len(values))
        query = f"INSERT INTO {self.__class__.__name__.lower()} ({', '.join(fields)}) VALUES ({placeholders})"
        Database.execute(query, values)

    @classmethod
    def all(cls):
        rows = Database.execute(f"SELECT * FROM {cls.__name__.lower()}").fetchall()
        return rows



def get_column_type(field):
    if field.__class__.__name__ == "IntField":
        return "INT"
    elif field.__class__.__name__ == "CharField":
        return f"VARCHAR({field.max_length})"
    else:
        raise TypeError("Unsupported field type for MySQL")



import mysql.connector
import psycopg2

class Database:
    connection = None
    cursor = None

    @classmethod
    def connect(cls, engine, **kwargs):
        if engine == "mysql":
            cls.connection = mysql.connector.connect(**kwargs)
        elif engine == "yugabyte":
            cls.connection = psycopg2.connect(**kwargs)
        cls.cursor = cls.connection.cursor()

    @classmethod
    def execute(cls, query, params=None):
        cls.cursor.execute(query, params or [])
        cls.connection.commit()
        return cls.cursor



class Field:
    def __init__(self, field_type, primary_key=False, foreign_key=None):
        self.type = field_type
        self.primary_key = primary_key
        self.foreign_key = foreign_key

class IntField(Field):
    def __init__(self, **kwargs):
        super().__init__('INT', **kwargs)

class CharField(Field):
    def __init__(self, max_length=255, **kwargs):
        super().__init__(f'VARCHAR({max_length})', **kwargs)

class FloatField(Field):
    def __init__(self, **kwargs):
        super().__init__('FLOAT', **kwargs)

class ForeignKeyField(Field):
    def __init__(self, ref_table):
        super().__init__('INT', foreign_key=ref_table)
